==> rill-0.0.624.tgz: package/client/util/browser.js <==
/** Rill v0.0.624 https://www.npmjs.com/package/rill License: MIT*/
"use strict";
var browser, element, expect, handleForm, handleLink, handleURLChange, handler, hashReg, parseURL, referrer, serialize;

serialize = require("@rill/form-json");

parseURL = require("@rill/parse-url");

expect = require("../util/expect");

referrer = document.referrer;

hashReg = /#.+$/;

handler = function() {};

element = null;

browser = {

  /*
  	 * Listen to all url change events on a dom element and trigger the server callback.
  	 *
  	 * @param {DOMNode} element
  	 * @api private
   */
  listen: function(entity) {
    if (entity == null) {
      entity = global;
    }
    expect(element == null, "Can only run one server at a time.");
    element = entity;
    global.addEventListener("DOMContentLoaded", handleURLChange);
    global.addEventListener("popstate", handleURLChange);
    element.addEventListener("submit", handleForm);
    element.addEventListener("click", handleLink);
  },

  /*
  	 * Add a handler to listen to url changes.
  	 *
  	 * @param {Function} fn
  	 * @api private
   */
  createServer: function(fn) {
    handler = fn;
    return browser;
  },

  /*
  	 * Trigger the registered handler to navigate to a given url.
  	 *
  	 * @param {String} url
  	 * @param {Object} options
  	 * @api private
   */
  goTo: function(url, arg) {
    var body, files, headers, method, ref, replaceState, title;
    ref = arg != null ? arg : {}, method = ref.method, headers = ref.headers, body = ref.body, files = ref.files, title = ref.title, replaceState = ref.replaceState;
    if (title == null) {
      title = "";
    }
    if (headers == null) {
      headers = {};
    }
    headers.referer = referrer;
    referrer = url;
    return handler({
      url: url,
      body: body,
      files: files,
      method: method != null ? method : "GET",
      headers: headers
    }).then(function(arg1) {
      var hash, redirected, ref1, ref2;
      redirected = (arg1 != null ? arg1 : {}).redirected;
      if (!redirected) {

        /*
        				 * When navigating a user will be brought to the top of the page.
        				 * If the urls contains a hash that is the id of an element (a target) then the target will be scrolled to.
        				 * This is similar to how browsers handle page transitions natively.
         */
        if (hash = (ref1 = url.match(hashReg)) != null ? ref1[0].slice(1) : void 0) {
          if ((ref2 = document.getElementById(hash)) != null) {
            ref2.scrollIntoView();
          }
        } else {
          global.scrollTo(0, 0);
        }
        if (replaceState) {
          history.replaceState(null, title, url);
        } else {
          history.pushState(null, title, url);
        }
      }
    });
  }
};


/*
 * Handle an event that changed the url (popstate or page load)
 *
 * @param {Object} event
 */

handleURLChange = function(event) {
  return browser.goTo(location.href, {
    method: "GET",
    replaceState: true
  });
};


/*
 * Handle intercepting link clicks to update the url.
 *
 * @param {Object} event
 */

handleLink = function(event) {
  var el, host, protocol, ref, url;
  if (event.defaultPrevented || event.metaKey || event.ctrlKey || event.shiftKey || event.button !== 0) {
    return;
  }
  el = event.target;
  while ((el != null) && el.nodeName !== "A") {
    el = el.parentNode;
  }
  if (el == null) {
    return;
  }
  if (el.target && el.target !== "_self") {
    return;
  }
  if (Boolean(el.attributes.download)) {
    return;
  }
  url = el.href;
  ref = parseURL(url, el), protocol = ref.protocol, host = ref.host;
  if (location.protocol.indexOf(protocol) === -1 || location.host !== host) {
    return;
  }
  if (el.rel && /(?:^|\s+)external(?:\s+|$)/.test(el.rel)) {
    return;
  }
  browser.goTo(url);
  return event.preventDefault();
};


/*
 * Handle intercepting forms to update the url.
 *
 * @param {Object} event
 */

handleForm = function(event) {
  var body, el, files, host, method, protocol, ref, ref1, url;
  if (event.defaultPrevented || event.metaKey || event.ctrlKey || event.shiftKey) {
    return;
  }
  el = event.target;
  while ((el != null) && el.nodeName !== "FORM") {
    el = el.parentNode;
  }
  if (el == null) {
    return;
  }
  if (el.target && el.target !== "_self") {
    return;
  }
  url = el.action;
  ref = parseURL(url, location), protocol = ref.protocol, host = ref.host;
  if (location.protocol.indexOf(protocol) === -1 || location.host !== host) {
    return;
  }
  if (el.rel && /(?:^|\s+)external(?:\s+|$)/.test(el.rel)) {
    return;
  }
  ref1 = serialize(el), body = ref1.body, files = ref1.files;
  method = (el.getAttribute("method") || el.method).toUpperCase();
  if (!el.hasAttribute("data-noreset")) {
    el.reset();
  }
  if (method === "GET") {
    url = (url.split("?")[0]) + "?" + (JSON.stringify(body));
    body = {};
    files = [];
  }
  browser.goTo(url, {
    method: method,
    body: body,
    files: files
  });
  return event.preventDefault();
};

module.exports = browser;

==> rill-0.0.626.tgz: package/client/util/browser.js <==
/** Rill v0.0.626 https://www.npmjs.com/package/rill License: MIT*/
"use strict";
var browser, element, expect, handleForm, handleLink, handleURLChange, handler, hashReg, parseURL, referrer, serialize;

serialize = require("@rill/form-json");

parseURL = require("@rill/parse-url");

expect = require("../util/expect");

referrer = document.referrer;

hashReg = /#.+$/;

handler = function() {};

element = null;

browser = {

  /*
  	 * Listen to all url change events on a dom element and trigger the server callback.
  	 *
  	 * @param {DOMNode} element
  	 * @api private
   */
  listen: function(entity) {
    if (entity == null) {
      entity = global;
    }
    expect(element == null, "Can only run one server at a time.");
    element = entity;
    global.addEventListener("DOMContentLoaded", handleURLChange);
    global.addEventListener("popstate", handleURLChange);
    element.addEventListener("submit", handleForm);
    element.addEventListener("click", handleLink);
  },

  /*
  	 * Add a handler to listen to url changes.
  	 *
  	 * @param {Function} fn
  	 * @api private
   */
  createServer: function(fn) {
    handler = fn;
    return browser;
  },

  /*
  	 * Trigger the registered handler to navigate to a given url.
  	 *
  	 * @param {String} url
  	 * @param {Object} options
  	 * @api private
   */
  goTo: function(url, arg) {
    var body, files, headers, method, ref, replaceState, title;
    ref = arg != null ? arg : {}, method = ref.method, headers = ref.headers, body = ref.body, files = ref.files, title = ref.title, replaceState = ref.replaceState;
    if (title == null) {
      title = "";
    }
    if (headers == null) {
      headers = {};
    }
    headers.referer = referrer;
    referrer = url;
    return handler({
      url: url,
      body: body,
      files: files,
      method: method != null ? method : "GET",
      headers: headers
    }).then(function(arg1) {
      var hash, redirected, ref1, ref2;
      redirected = (arg1 != null ? arg1 : {}).redirected;
      if (!redirected) {

        /*
        				 * When navigating a user will be brought to the top of the page.
        				 * If the urls contains a hash that is the id of an element (a target) then the target will be scrolled to.
        				 * This is similar to how browsers handle page transitions natively.
         */
        if (hash = (ref1 = url.match(hashReg)) != null ? ref1[0].slice(1) : void 0) {
          if ((ref2 = document.getElementById(hash)) != null) {
            ref2.scrollIntoView({
              block: "start",
              behavior: "smooth"
            });
          }
        } else {
          global.scrollTo(0, 0);
        }
        if (replaceState) {
          history.replaceState(null, title, url);
        } else {
          history.pushState(null, title, url);
        }
      }
    });
  }
};


/*
 * Handle an event that changed the url (popstate or page load)
 *
 * @param {Object} event
 */

handleURLChange = function(event) {
  return browser.goTo(location.href, {
    method: "GET",
    replaceState: true
  });
};


/*
 * Handle intercepting link clicks to update the url.
 *
 * @param {Object} event
 */

handleLink = function(event) {
  var el, host, protocol, ref, url;
  if (event.defaultPrevented || event.metaKey || event.ctrlKey || event.shiftKey || event.button !== 0) {
    return;
  }
  el = event.target;
  while ((el != null) && el.nodeName !== "A") {
    el = el.parentNode;
  }
  if (el == null) {
    return;
  }
  if (el.target && el.target !== "_self") {
    return;
  }
  if (Boolean(el.attributes.download)) {
    return;
  }
  url = el.href;
  ref = parseURL(url, el), protocol = ref.protocol, host = ref.host;
  if (location.protocol.indexOf(protocol) === -1 || location.host !== host) {
    return;
  }
  if (el.rel && /(?:^|\s+)external(?:\s+|$)/.test(el.rel)) {
    return;
  }
  browser.goTo(url);
  return event.preventDefault();
};


/*
 * Handle intercepting forms to update the url.
 *
 * @param {Object} event
 */

handleForm = function(event) {
  var body, el, files, host, method, protocol, ref, ref1, url;
  if (event.defaultPrevented || event.metaKey || event.ctrlKey || event.shiftKey) {
    return;
  }
  el = event.target;
  while ((el != null) && el.nodeName !== "FORM") {
    el = el.parentNode;
  }
  if (el == null) {
    return;
  }
  if (el.target && el.target !== "_self") {
    return;
  }
  url = el.action;
  ref = parseURL(url, location), protocol = ref.protocol, host = ref.host;
  if (location.protocol.indexOf(protocol) === -1 || location.host !== host) {
    return;
  }
  if (el.rel && /(?:^|\s+)external(?:\s+|$)/.test(el.rel)) {
    return;
  }
  ref1 = serialize(el), body = ref1.body, files = ref1.files;
  method = (el.getAttribute("method") || el.method).toUpperCase();
  if (!el.hasAttribute("data-noreset")) {
    el.reset();
  }
  if (method === "GET") {
    url = (url.split("?")[0]) + "?" + (JSON.stringify(body));
    body = {};
    files = [];
  }
  browser.goTo(url, {
    method: method,
    body: body,
    files: files
  });
  return event.preventDefault();
};

module.exports = browser;
